package net.wanji.opt.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.GreenwaveHistoryMapper;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.dto.report.StrategyGreenOptHistDTO;
import org.joda.time.DateTime;
import org.joda.time.Seconds;
import org.joda.time.format.DateTimeFormat;
import org.springframework.beans.factory.annotation.Configurable;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author fengyi
 * @date 2023/1/13
 * @description
 */
//@Component
@Configurable
@EnableScheduling
@Slf4j
@Profile("!dev")
public class OptimizeStatisticTask {


    @Resource
    StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    GreenwaveHistoryMapper historyMapper;
 

    /**
     * 干线策略优化时长计算
     */
//    @Scheduled(cron查询上周优化记录
    public void optimizeGreenWaveDurationCal() {
        long st = System.currentTimeMillis();
        try {
            long et = System.currentTimeMillis();
            List<StrategyGreenOptHistDTO> durationList = new ArrayList<>();

            List<StrategyGreenOptHistDTO> list = strategyGreenOptHistMapper.findGreenWaveOptHist();
            Map<Integer, List<StrategyGreenOptHistDTO>> groupByGreenId = list.stream().collect(Collectors.groupingBy(o -> o.getGreenId()));

            for (Map.Entry<Integer, List<StrategyGreenOptHistDTO>> entry : groupByGreenId.entrySet()) {

                List<StrategyGreenOptHistDTO> value = entry.getValue();
                value = value.stream().sorted(Comparator.comparing(StrategyGreenOptHistDTO::getControlTime)).collect(Collectors.toList());
                int i = 0;
                for (StrategyGreenOptHistDTO result : value) {
                    int seconds = 900;
                    if (i != value.size() - 1) {
                        if (value.get(i + 1).getControlMethod() == -1) {
                            //下一条记录为取消状态，当前取消时间-上次下方时间
                            seconds = Seconds.secondsBetween(new DateTime(result.getControlTime()), new DateTime(value.get(i + 1).getControlTime())).getSeconds();
                            //超过设定的ControlDuration取ControlDuration
                            if (seconds > result.getControlDuration()) {
                                seconds = result.getControlDuration();
                            }
                            //log.info("greenId={}：干线绿波方案持续时间计算，下一条记录为取消状态 开始时间：{},截止时间：{},持续时长：{}秒",
//                                    result.getGreenId(),
//                                    new DateTime(result.getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                    new DateTime(value.get(i + 1).getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                    seconds);
                        } else if(result.getControlMethod() != -1) {
                            //两条记录之前没有取消状态，取ControlDuration
                            seconds = result.getControlDuration();
                            //log.info("greenId={}：干线绿波方案持续时间计算，两条记录之前没有取消状态 开始时间：{},截止时间：{},持续时长：{}秒",
//                                    result.getGreenId(),
//                                    new DateTime(result.getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                    new DateTime(value.get(i + 1).getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                    seconds);
                        }
                    } else if (result.getControlMethod() != -1) {
                        //最后一天记录不为-1取消状态
                        seconds = Seconds.secondsBetween(new DateTime(result.getControlTime()), new DateTime(new Date())).getSeconds();
                        if (seconds > result.getControlDuration()) {
                            seconds = result.getControlDuration();
                        }
                        //log.info("greenId={}：干线绿波方案持续时间计算，最后一天记录不为取消状态 开始时间：{},截止时间：{},持续时长：{}秒",
//                                result.getGreenId(),
//                                new DateTime(result.getControlTime()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                new DateTime(new Date()).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")),
//                                seconds);

                    }
                    result.setDuration(seconds);
                    //优化控制时段-开始时间
                    String controlStartTime = result.getControlTime();
                    //优化控制时段-截止时间
                    String controlEndTime = new DateTime(result.getControlTime()).plusSeconds(seconds).toString(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));

                    durationList.add(result);
                    i++;
                }
            }
            if (!durationList.isEmpty()) {

            }
            log.info("更新干线策略持续时间据耗时：{}ms，size:{}", et - st, durationList.size());

        } catch (Exception e) {
            log.error("", e);
        }
    }


}
